#include <Keyboard.h>
#include <EEPROM.h>

/* Programmable Foot Switch PAVR-20190416-01

by Russ Pitman and Steve Smith

V1.02 27/05/2019

Construction details at www.ProjectAVR.com

The device reads an 8-way DIP switch connected to
Arduino pins 2 to 9 set by the user and converts
that to a keyboard code. This code is then saved
to one of four memory locations selected by a four
way rotary switch, when the 'program' button
is depressed.

Upon a press of the foot switch or test button,
the keyboard code indicated by the rotary switch is
sent via USB to the host machine as a keypress.  

ASCII Character to Binary Table
-------------------------------

Char  Bin         | Char  Bin         | Char  Bin
----- ----      --  ----- -----     --  ----- -----
spc   0010 0000   | @     0100 0000   | `     0110 0000
!     0010 0001   | A     0100 0001   | a     0110 0001
"     0010 0010   | B     0100 0010   | b     0110 0010
#     0010 0011   | C     0100 0011   | c     0110 0011
$     0010 0100   | D     0100 0100   | d     0110 0100
%     0010 0101   | E     0100 0101   | e     0110 0101
&     0010 0110   | F     0100 0110   | f     0110 0110
'     0010 0111   | G     0100 0111   | g     0110 0111
(     0010 1000   | H     0100 1000   | h     0110 1000
)     0010 1001   | I     0100 1001   | i     0110 1001
*     0010 1010   | J     0100 1010   | j     0110 1010
+     0010 1011   | K     0100 1011   | k     0110 1011
,     0010 1100   | L     0100 1100   | l     0110 1100
-     0010 1101   | M     0100 1101   | m     0110 1101
.     0010 1110   | N     0100 1110   | n     0110 1110
/     0010 1111   | O     0100 1111   | o     0110 1111
0     0011 0000   | P     0101 0000   | p     0111 0000
1     0011 0001   | Q     0101 0001   | q     0111 0001
2     0011 0010   | R     0101 0010   | r     0111 0010
3     0011 0011   | S     0101 0011   | s     0111 0011
4     0011 0100   | T     0101 0100   | t     0111 0100
5     0011 0101   | U     0101 0101   | u     0111 0101
6     0011 0110   | V     0101 0110   | v     0111 0110
7     0011 0111   | W     0101 0111   | w     0111 0111
8     0011 1000   | X     0101 1000   | x     0111 1000
9     0011 1001   | Y     0101 1001   | y     0111 1001
:     0011 1010   | Z     0101 1010   | z     0111 1010
;     0011 1011   | [     0101 1011   | {     0111 1011
<     0011 1100   | \     0101 1100   | |     0111 1100
=     0011 1101   | ]     0101 1101   | }     0111 1101
>     0011 1110   | ^     0101 1110   | ~     0111 1110
?     0011 1111   | _     0101 1111

*/

// Global Variables
//-----------------
//

const int PROG_Pin          = 18; // A0
const int LED_Pin           = 19; // A1
const int FOOT_Pin          = 20; // A2
const int ENABLE_Pin        = 21; //A3
int       DIP_Switch[]      = {0,0,0,0,0,0,0,0};
int       DIP_Pins[]        = {9,8,7,6,5,4,3,2};
int       DIP_Value         = 0;
int       DIP_Tmp           = 0;
int       FOOT_State        = 0;
int       ENABLE_State      = 0;

int       StoredRotaryPosn  = 0;
int       ROT_Pins[]        = {10,14,15,16};

String    ProgString        = "PROGRAM ";
String    ProgDisplay       = "";

char      KeyRead;

// Application Initialisation (setup)
//===================================
//
void setup() {
  
  // Init DIP Pins (2 - 9) as INPUT_PULLUP
  // -------------------------------------
  for (int Pin_No = 0; Pin_No <= 7; Pin_No++) {
    pinMode(DIP_Pins[Pin_No],INPUT_PULLUP);
  }

    // Init Rotary Switch Pins (10,14,15,16) as INPUT_PULLUP
  // -----------------------------------------------------
  for (int Pin_No = 0; Pin_No <= 3; Pin_No++) {
    pinMode(ROT_Pins[Pin_No],INPUT_PULLUP);
  }

  // Init Program Pin (18) as INPUT_PULLUP
  // -------------------------------------
  pinMode(PROG_Pin, INPUT_PULLUP);

  // Init LED Pin (19) as OUTPUT
  // ---------------------------
  pinMode(LED_Pin, OUTPUT);

  // Init FOOT Pin (20) as INPUT_PULLUP
  // ----------------------------------
  pinMode(FOOT_Pin, INPUT_PULLUP);

  // Initialise ENABLE Pin (21) as INPUT_PULLUP
  // ----------------------------------------
  pinMode(ENABLE_Pin, INPUT_PULLUP);
    
  // Allow time 
  //-----------
  delay(50);
}


// Application Main (loop)
//========================
//
void loop() {

  // Display LOOP
  SEND_SerialData(0,"LOOP",false);
  
  if (digitalRead(PROG_Pin) == LOW) {

    SEND_SerialData(0, "PROGRAM", false);
    
    ReadDIP();    // Read DIP and calculate

    ReadRotary(); // Read Rotary 
      
    // Store character in EEPROM
    EEPROM.write(StoredRotaryPosn, DIP_Value);
    
        // Display "PROGRAM **"
        //
        ProgDisplay = ProgString + StoredRotaryPosn;
        SEND_SerialData(0, "PROGRAM", false);
        FlashLED(3);    // Flash LED 3 times
    
        // Check Program switch released
        //
        while (digitalRead(PROG_Pin) == LOW) {
        // do nothing until Program switch released
        }
  }

  FOOT_State = digitalRead(FOOT_Pin);
  
  if (FOOT_State == HIGH) {
      
    ReadRotary();

    KeyRead = char(EEPROM.read(StoredRotaryPosn));
    SEND_KeyboardData(KeyRead);

    FlashLED(1);    // Flash LED 1 time

    // Display "FOOT"
    //
        SEND_SerialData(0,"FOOT",false);  
  
  }
    
}



//=====================
// Additional functions
//=====================

//------------------
// Read DIP Switches
//------------------
//
// Read DIP switch position and store correctly.
// Also calculate DIP_Value
//
void ReadDIP(){

  DIP_Tmp   = 1;  // Multiplication Factor
  DIP_Value = 0;  // Overall value

  // Read 8-way DIP switch and store
    //--------------------------------
  for (int DIPPos = 0; DIPPos <= 7; DIPPos++) {

    DIP_Switch[DIPPos] = !digitalRead(DIP_Pins[DIPPos]);
      
    DIP_Value = DIP_Value + (DIP_Switch[DIPPos] * DIP_Tmp);
    DIP_Tmp   = DIP_Tmp * 2;
    
  } 
}


//-------------------
// Read Rotary Switch
//-------------------
//
// Read Rotary switch position and store StoredRotaryPosn
//
void ReadRotary(){

  // Read rotary switch and store
  //--------------------------------
  for (int RotaryPos = 0; RotaryPos <= 3; RotaryPos++) {
    if (digitalRead (ROT_Pins[RotaryPos]) == LOW){
      StoredRotaryPosn = RotaryPos;
    }
  }
}


//-------------
// LED Flashing
//-------------
//
// Enter the number of times that you want the LED
// to flash 1,2,3.  Entering a number ABOVE 3 will
// give you a single LONG flash
//
void FlashLED(int Flashes){

  if ( Flashes < 4 ) {
    
    for ( int FlshCnt = 1; FlshCnt <= Flashes; FlshCnt++) {
      digitalWrite(LED_Pin, HIGH);
      delay (50);
          digitalWrite(LED_Pin, LOW);
          delay (150);
    }
  }
  else {
    digitalWrite(LED_Pin, HIGH);
    delay (1000);
        digitalWrite(LED_Pin, LOW);
        delay (150);
  }
}


//-----------------
// SEND Serial Data
//-----------------
//
// Send and integer Value or String Word to the Serial 
// Port for display, if isVal is true then Value is
// displayed else Word.
//
void SEND_SerialData(int Value, String Word, bool isVal) {
  
  Serial.begin(9600);

  if (isVal) {
    Serial.print(Value);
  }
  else {
    Serial.print(Word);
  }

  Serial.println();
  Serial.end();

}


//-------------------
// SEND Keyboard Data
//-------------------
//
// Send keypress data to the keyboard
//
void SEND_KeyboardData(char Key){

  // Initialize Keyboard 
  Keyboard.begin();
     
  // Press Key selected 
  Keyboard.press(Key);
      
  // Allow time 
  delay(50);
      
  // Release keypress
  Keyboard.releaseAll();

  // Allow time 
  delay(50);
  Keyboard.end();
}
